package main

import (
	"flag"
	"os"
	"testing"

	log "github.com/sirupsen/logrus"

	"code.fbi.h-da.de/danet/proto-kms/kms"
	"github.com/google/uuid"
)

func TestMain(m *testing.M) {
	// run a self-test if set to yes
	var selfTesting bool
	// my own Name
	var udpQL1AddrString string
	var ql1Name string
	// a name for potential self-testing
	var udpQL2AddrString string
	var ql2Name string
	// Setup flags that allows to initially provide a address to a second
	// quantumlayer so generated keys can be fetched. This is mainly to simplify
	// the initial setup process.

	flag.BoolVar(&selfTesting, "selftesting", true,
		"Will perform self-testing with second quantum link if set to true")

	flag.StringVar(&udpQL1AddrString, "my-address", "[::1]:50900",
		"The UDP address for my quantumlayer so keys can be fetched")

	flag.StringVar(&ql1Name, "my-name", "ekms-ql1",
		"The name of the quantumlayer")

	flag.StringVar(&udpQL2AddrString, "remote-address", "[::1]:50901",
		"The UDP address to a second quantumlayer so keys can be fetched")

	flag.StringVar(&ql2Name, "remote-name", "ekms-ql2",
		"The name of the quantumlayer")

	// Parse the provided flags
	flag.Parse()

	log.Println("Welcome to the proto-kms called: ", ql1Name)

	if selfTesting == true {
		log.Infof("%s in self-testing mode", ql1Name)
		go emulatedKMS("leftKMS", "[::1]:50900", "[::1]:50930", "[::1]:50931", true)
		emulatedKMS("rightKMS", "[::1]:50901", "[::1]:50931", "[::1]:50930", false)

	} else {
		//	log.Infof("%s in regular mode of operation", ql1Name)
		//	emulatedKMS(ql1Name, udpQL1AddrString, udpQL2AddrString)
	}
}

func emulatedKMS(myName, myUDPAddr, myInterComAddr, peerInterComAddr string, master bool) {
	// Attach to eKMS
	emuKMS := kms.NewEKMS(myName, uuid.New(), os.Stdout, log.TraceLevel, false, myInterComAddr)

	eqm := kms.NewEmulatedQuantumModule("::1", os.Stdout, log.TraceLevel, false)

	// Fire up Quantum LinK
	err := emuKMS.AddQuantumElement(eqm)
	if err != nil {
		log.Println("failed to add emulated quantum module")
		return
	}

	_, err = emuKMS.AddPeer(myName, peerInterComAddr, eqm)
	if err != nil {
		log.Println("PEERERROR: ", err)
	}

	// Start the SDN/management and key retrieval interface
	go kms.StartETSI(myUDPAddr, emuKMS)

	for {
	}
}
